/*** DX11 CBuffer (Constant Buffer) Manager class, Cristiaan Kop	 upd 27-12-2016 ***/
/*** ============================================================================== ***/
/*** NAMESPACE: Crealysm::RENDERER (DX11)											***/
/*** cpp file with the DX11CBufferMgr class implementation	   						***/

#include "RENDERER/DX11/DX11cbuffermgr.h"

#include <assert.h>

using namespace Crealysm::COMMON;


namespace Crealysm { namespace RENDERER
{

/**************************************************************************************/
/***								CONSTRUCTOR										***/
/*** ==> usage: when creating a DX11CBufferMgr object 								***/
/*** ==> sets all variables in the class object to initial/ passed values			***/
/**************************************************************************************/

DX11CBufferMgr::DX11CBufferMgr(ID3D11Device *pDevice) : mDevice(pDevice)
{	
	assert(mDevice);
	
	mFreeList.reserve(MAX_CB);
	for(size_t i=0;i<=MAX_CB;++i) mFreeList.push_back(MAX_CB - i);

	CLOG(DEBUG, "DX11RENDERER") << debug_cb_mgr_created.c_str();
}

/**************************************************************************************/
/***								DESTRUCTOR										***/
/*** ==> usage: when DX11CBufferMgr object is not needed anymore					***/
/*** ==> releases stuff, COM objects, memory etc. 									***/
/**************************************************************************************/

DX11CBufferMgr::~DX11CBufferMgr()
{
	// nothing yet
}

/**************************************************************************************/
/***									RELEASE										***/
/*** ==> usage: to free up a CB and make room for another one						***/
/*** ==> sets the requested CB to nullptr for reuse									***/
/**************************************************************************************/

bool DX11CBufferMgr::Release(const uint pHandle)
{
	if(pHandle >= mCBuffers.size()) return false;

	mFreeList.push_back(pHandle);

	mCBuffers[pHandle].Buffer = nullptr;
	mCBuffers[pHandle].DataSize = 0;

	CLOG(DEBUG, "DX11RENDERER") << debug_cbuffer_released.c_str() << pHandle;
	return true;
}




/**************************************************************************************/
/***								GET CBUFFER PTR							  CONST	***/
/*** ==> usage: to retrieve a pointer to a specific Constant Buffer					***/
/*** ==> returns the (const) pointer									 			***/
/**************************************************************************************/

ID3D11Buffer* DX11CBufferMgr::GetCBufferPtr(const uint pHandle) const
{
	if(pHandle >= mCBuffers.size()) return nullptr;
	return mCBuffers[pHandle].Buffer;
}


}
}