/*** DX11 Device class, Cristiaan Kop							 updated 02-01-2017 ***/
/*** ============================================================================== ***/
/*** NAMESPACE: Crealysm::RENDERER (DX11)											***/
/*** header file for the DX11Device class interface									***/

#ifndef DX11DEVICE_H
#define DX11DEVICE_H

#include <d3d11.h>
#include <atlbase.h>

#include "RENDERER/INTERFACES/idevice.h"
#include "RENDERER/INTERFACES/irendertarget.h"
#include "RENDERER/crenderersettings.h"
	
/**** NOTE
BACKBUFFER_FORMAT -> DXGI_FORMAT_R8G8B8A8_UNORM_SRGB		
// sRGB -> linear conversion is done automatically by D3D11 (gamma correct)
// all input diffuse/ color maps and material/ lighting values should be in sRGB too
*****/


namespace Crealysm { namespace RENDERER
{

/**************************************************************************************/
/***								DX11DEVICE CLASS								***/
/*** Manages the GPU device for DX11												***/
/**************************************************************************************/

class DX11Device : public IDevice
{
public:
	DX11Device(HWND pHwnd, CRendererSettings *pSettings, const CR_BUFFER_FORMAT pBackBufferFormat, const CR_BUFFER_FORMAT pDepthBufferFormat);
	~DX11Device();

	// Abstracted
	void* GetAPIDevicePtr()			const;
	void* GetAPIContextPtr()		const;

	bool Startup();
	bool UpdateFullscreenState();
	bool Resize();
	bool CheckAASupport(const CR_BUFFER_FORMAT pFormat, const uint pNrSamples);

	bool SetRenderTarget(const IRenderTarget &pRT);
	void ResetRenderTargets();
	bool Present(const IRenderTarget &pRT);

	bool ClearRTView(const IRenderTarget &pRT, const float pColor[4]);
	bool ClearDepthView();

	bool SetViewport(const CViewport &pViewport);

private:
	// Abstracted
	bool Create();

	bool CreateSwapchain();
	bool ResizeSwapchain();
	bool CreateDepthBuffer();

	// DX11 Specific
	CComPtr<ID3D11Device>				mDevice;
	CComPtr<ID3D11DeviceContext>		mDeviceContext;
	CComPtr<IDXGISwapChain>				mSwapChain;
	CComPtr<ID3D11DepthStencilView>		mDepthStencilView;

	DXGI_FORMAT							mDXGIBackBufferFormat;
	DXGI_FORMAT							mDXGIDepthBufferFormat;
};


}
}

#endif