/*** DX11 RenderTarget class, Cristiaan Kop						 updated 01-01-2017 ***/
/*** ============================================================================== ***/
/*** NAMESPACE: Crealysm::RENDERER (DX11)											***/
/*** header file for the DX11RenderTarget class interface							***/

#ifndef DX11RENDERTARGET_H
#define DX11RENDERTARGET_H

#include <d3d11.h>
#include <atlbase.h>

#include "RENDERER/INTERFACES/irendertarget.h"
#include "RENDERER/INTERFACES/idevice.h"


namespace Crealysm { namespace RENDERER
{

/**************************************************************************************/
/***								DX11RENDERTARGET CLASS							***/
/*** Manages a DX11 rendertarget													***/
/**************************************************************************************/

class DX11RenderTarget : public IRenderTarget
{
public:
	DX11RenderTarget(IDevice *pDevice, const uint pWidth, const uint pHeight, const CR_BUFFER_FORMAT pBufferFormat, const uint pNrAASamples, const uint pAAQuality, const bool pUseBaseConfig);
	~DX11RenderTarget();

	void* GetAPIBufferPtr()		const;
	void* GetAPIViewPtr()		const;

	bool Resize(const uint pWidth, const uint pHeight);
	bool ChangeAA(const uint pNrAASamples, const uint pAAQuality);
	bool Change(const uint pWidth, const uint pHeight, const uint pNrAASamples, const uint pAAQuality);

private:
	bool Create();

	bool CreateBuffer();
	
	CComPtr<ID3D11Texture2D>		mBuffer;
	CComPtr<ID3D11RenderTargetView>	mView;

	DXGI_FORMAT						mDXGIFormat;
};


}
}

#endif