/*** DX11 shaderpack class, Cristiaan Kop						 updated 27-12-2016 ***/
/*** ============================================================================== ***/
/*** NAMESPACE: Crealysm::RENDERER (DX11)											***/
/*** header file for the DX11ShaderPack class interface				     		    ***/

#ifndef DX11SHADERPACK_H
#define DX11SHADERPACK_H

#include <d3d11.h>
#include <string>
#include <vector>
#include <atlbase.h>

#include "RENDERER/DX11/DX11helper.h"


namespace Crealysm { namespace RENDERER 
{
	
/**************************************************************************************/
/***								DX11 SHADERPACK									***/
/*** stores a shaderpack (with PS/VS combo, defines etc.							***/
/*** copy/move constructor + move assigned = allowed, because std::vector in mgr	***/		
/**************************************************************************************/

class DX11ShaderPack
{
public:
	DX11ShaderPack();
	~DX11ShaderPack();

	DX11ShaderPack& operator=(const DX11ShaderPack& other) = delete;	// copy assignment: not allowed

	uint GetIdentifier()								const;
	const std::string &GetIdentifierText()				const;

	bool HasVS()										const;
	bool HasPS()										const;
	bool HasGS()										const;

	const ID3DBlob* GetVSShaderBlob()					const;

	ID3D11VertexShader*		GetVSPtr()					const;
	ID3D11PixelShader*		GetPSPtr()					const;
	ID3D11GeometryShader*	GetGSPtr()					const;
	
	uint GetInputLayoutID()								const;
	const std::string& GetInputLayoutMD5Checksum()		const;

	bool CreateShaders(ID3D11Device *pDevice, const std::string &pShaderFolder, const std::string &pFilename, const uint pPackId, const std::string &pTarget);
	bool CreateFallback(ID3D11Device *pDevice, const std::string &pTarget, const std::string &pRootFolder);

	CComPtr<ID3D11InputLayout> GetInputLayout(ID3D11Device *pDevice);
	void SetInputLayoutId(const uint pId);

private:
	std::vector<D3D_SHADER_MACRO> GetD3dDefines()		const;

	bool LoadMetadata(const std::string &pFilename, const std::string &pShaderFolder, const uint pPackId);
	bool CreateInputLayoutMD5Checksum();
	
	bool CreateElementDesc();
	bool CreateDefines(const std::string &pDefineLine);

	bool CompileAndCreateVS(ID3D11Device *pDevice, const std::string &pTarget);
	bool CompileAndCreatePS(ID3D11Device *pDevice, const std::string &pTarget);
	bool CompileAndCreateGS(ID3D11Device *pDevice, const std::string &pTarget);

	std::vector<DX11_VS_INPUT_ELEMENTDESC>	mElementDesc;
	std::vector<C_SHADER_MACRO>				mDefines;
	std::vector<D3D_SHADER_MACRO>			mD3dDefines;

	CComPtr<ID3D11VertexShader>			mVS;
	CComPtr<ID3D11PixelShader>			mPS;
	CComPtr<ID3D11GeometryShader>		mGS;

	CComPtr<ID3DBlob>					mVSShaderBlob;

	uint					mFlags;

	std::string				mInputLayoutMD5Checksum;
	uint					mInputLayoutId;
	uint					mIdentifier;
	std::string				mIdentifierText;

	std::string				mVSEntryPoint;
	std::string				mPSEntryPoint;
	std::string				mGSEntryPoint;

	std::string				mVSFilename;
	std::string				mPSFilename;
	std::string				mGSFilename;
};

}
}

#endif