/*** IDevice class, Cristiaan Kop								 updated 02-01-2017 ***/
/*** ============================================================================== ***/
/*** NAMESPACE: Crealysm::RENDERER													***/
/*** header file for the IDevice class interface									***/

#ifndef IDEVICE_H
#define IDEVICE_H

#include <Windows.h>

#include "RENDERER/INTERFACES/itypes.h"
#include "RENDERER/crenderersettings.h"
#include "RENDERER/cbufferformats.h"
#include "RENDERER/cviewport.h"

#include "RENDERER/INTERFACES/irendertarget.h"

namespace Crealysm { namespace RENDERER
{

/**************************************************************************************/
/***								IDEVICE CLASS									***/
/*** Base class for GPU devices, API device class inherits from this 				***/
/**************************************************************************************/

class IDevice 
{
public:
	IDevice(HWND pHwnd, CRendererSettings *pSettings, const CR_BUFFER_FORMAT pBackBufferFormat, const CR_BUFFER_FORMAT pDepthBufferFormat);
	virtual ~IDevice();

	virtual void* GetAPIDevicePtr()		const = 0;
	virtual void* GetAPIContextPtr()	const = 0;

	virtual bool Startup() = 0;
	virtual bool UpdateFullscreenState() = 0;
	virtual bool Resize() = 0;
	virtual bool CheckAASupport(const CR_BUFFER_FORMAT pFormat, const uint pNrSamples) = 0;

	virtual bool SetRenderTarget(const IRenderTarget &pRT) = 0;
	virtual void ResetRenderTargets() = 0;
	virtual bool Present(const IRenderTarget &pRT) = 0;

	virtual bool ClearRTView(const IRenderTarget &pRT, const float pColor[4]) = 0;
	virtual bool ClearDepthView() = 0;

	virtual bool SetViewport(const CViewport &pViewport) = 0;

protected:
	virtual bool Create() = 0;

	virtual bool CreateSwapchain() = 0;
	virtual bool ResizeSwapchain() = 0;
	virtual bool CreateDepthBuffer() = 0;

	CRendererSettings	*mSettings;

	CR_BUFFER_FORMAT	mBackBufferFormat;
	CR_BUFFER_FORMAT	mDepthBufferFormat;

	HWND				mHwnd;
};


}
}

#endif