/*** IRenderTarget class, Cristiaan Kop							 updated 01-01-2017 ***/
/*** ============================================================================== ***/
/*** NAMESPACE: Crealysm::RENDERER													***/
/*** CPP file for the IRenderTarget class implementation							***/

#include "RENDERER/INTERFACES/irendertarget.h"


namespace Crealysm { namespace RENDERER
{

/**************************************************************************************/
/***								CONSTRUCTOR										***/
/*** ==> usage: when creating a IRenderTarget object								***/
/*** ==> makes the object 'valid'													***/
/**************************************************************************************/

IRenderTarget::IRenderTarget()
	: mWidth(0), mHeight(0), mBufferFormat(CR_FORMAT_UNKNOWN), mAASamples(1), mAAQuality(0), mAAEnabled(false), mUseBaseConfig(true), mDevicePtr(nullptr)	
{ 
	// nothing yet
}

IRenderTarget::IRenderTarget(IDevice *pDevice, const uint pWidth, const uint pHeight, const CR_BUFFER_FORMAT pCrBufferFormat, const uint pNrAASamples, const uint pAAQuality, const bool pUseBaseConfig)
	: mWidth(pWidth), mHeight(pHeight), mBufferFormat(pCrBufferFormat), mAASamples(pNrAASamples), mAAQuality(pAAQuality), mUseBaseConfig(pUseBaseConfig)
{
	mDevicePtr = pDevice;
}

/**************************************************************************************/
/***								DESTRUCTOR										***/
/*** ==> usage: when IRenderTarget object is not needed anymore						***/
/*** ==> releases stuff, objects, memory etc. 										***/
/**************************************************************************************/

IRenderTarget::~IRenderTarget()
{
	mDevicePtr = nullptr;
}


}
}