/*** IRenderTarget class, Cristiaan Kop							 updated 02-01-2017 ***/
/*** ============================================================================== ***/
/*** NAMESPACE: Crealysm::RENDERER													***/
/*** header file for the IRenderTarget class interface								***/

#ifndef IRENDERTARGET_H
#define IRENDERTARGET_H

#include "RENDERER/INTERFACES/itypes.h"
#include "RENDERER/cbufferformats.h"
#include "RENDERER/INTERFACES/idevice.h"


namespace Crealysm { namespace RENDERER
{

/**************************************************************************************/
/***							IRENDERTARGET CLASS									***/
/*** Base class for RenderTargets, API RT class inherits from this					***/
/**************************************************************************************/

class IRenderTarget 
{
public:
	IRenderTarget();
	IRenderTarget(IDevice *pDevice, const uint pWidth, const uint pHeight, const CR_BUFFER_FORMAT pBufferFormat, const uint pNrAASamples, const uint pAAQuality, const bool pUseBaseConfig);

	virtual ~IRenderTarget();

	virtual void* GetAPIBufferPtr()		const = 0;
	virtual void* GetAPIViewPtr()		const = 0;

	virtual bool Resize(const uint pWidth, const uint pHeight) = 0;
	virtual bool ChangeAA(const uint pNrAASamples, const uint pAAQuality) = 0;
	virtual bool Change(const uint pWidth, const uint pHeight, const uint pNrAASamples, const uint pAAQuality) = 0;

protected:
	virtual bool Create() = 0;

	uint				mWidth;
	uint				mHeight;
	CR_BUFFER_FORMAT	mBufferFormat;
	uint				mAASamples;
	uint				mAAQuality;

	bool				mAAEnabled;
	bool				mUseBaseConfig;

	IDevice				*mDevicePtr;
};


}
}

#endif